import gevent
import zmq.green as zmq
import random
import numpy as np

context = zmq.Context()

def dummy_AD():
    t = 0.0
    while True:
        x = np.arange(t, t+0.1, 0.001)
        y = np.sin(20*x) + np.random.normal(scale=0.1, size=len(x))
        pulse_count = random.randint(3, 10)
        idx = np.random.randint(0, len(x), pulse_count)
        y[idx] += np.random.normal(size=pulse_count)
        t += 0.1
        yield y

class DataServer(object):
    
    def __init__(self):
        self.pub_socket = context.socket(zmq.PUB)
        self.pub_socket.bind("tcp://*:7887")
        
        self.rep_socket = context.socket(zmq.REP)
        self.rep_socket.bind("tcp://*:7888")
        
        self.exit_event = gevent.event.Event()

    def pub_data(self):
        ad = dummy_AD()
        while True:
            self.pub_socket.send_pyobj(ad.next())
            gevent.sleep(0.1)
        
    def start_pub_data(self):
        self.publet = gevent.spawn(self.pub_data)
        
    def stop_pub_data(self):
        publet = getattr(self, "publet", None)
        if publet is not None:
            gevent.kill(publet)
            
    def exit(self):
        self.exit_event.set()

    def run(self):
        while not self.exit_event.is_set():
            cmd, args = self.rep_socket.recv_pyobj()
            print cmd, args
            ret = getattr(self, cmd)(*args)
            self.rep_socket.send_pyobj(ret)

print "start data server"            
server = DataServer()
server.run()